@@ -1,13 +1,13 @@
 # frozen_string_literal: true
 
 # Released under the MIT License.
-# Copyright, 2022, by Samuel Williams.
+# Copyright, 2022-2024, by Samuel Williams.
 
-require 'console'
+require "console"
 
-require_relative '../constants'
-require_relative '../input'
-require_relative '../response'
+require_relative "../constants"
+require_relative "../input"
+require_relative "../response"
 
 module Protocol
 	module Rack
@@ -17,6 +17,11 @@ module Protocol
 					self.new(app)
 				end
 				
+				def self.parse_file(...)
+					# This is the old interface, which was changed in Rack 3.
+					::Rack::Builder.parse_file(...).first
+				end
+				
 				# Initialize the rack adaptor middleware.
 				# @parameter app [Object] The rack middleware.
 				def initialize(app)
@@ -26,9 +31,9 @@ module Protocol
 				end
 				
 				def logger
-					Console.logger
+					Console
 				end
-
+				
 				# Unwrap raw HTTP headers into the CGI-style expected by Rack middleware.
 				#
 				# Rack separates multiple headers with the same key, into a single field with multiple lines.
@@ -40,7 +45,11 @@ module Protocol
 						http_key = "HTTP_#{key.upcase.tr('-', '_')}"
 						
 						if current_value = env[http_key]
-							env[http_key] = "#{current_value};#{value}"
+							if http_key == CGI::HTTP_COOKIE
+								env[http_key] = "#{current_value};#{value}"
+							else
+								env[http_key] = "#{current_value},#{value}"
+							end
 						else
 							env[http_key] = value
 						end
@@ -57,7 +66,7 @@ module Protocol
 				# @parameter request [Protocol::HTTP::Request] The incoming request.
 				# @parameter env [Hash] The rack `env`.
 				def unwrap_request(request, env)
-					if content_type = request.headers.delete('content-type')
+					if content_type = request.headers.delete("content-type")
 						env[CGI::CONTENT_TYPE] = content_type
 					end
 					
@@ -68,6 +77,16 @@ module Protocol
 					
 					self.unwrap_headers(request.headers, env)
 					
+					# For the sake of compatibility, we set the `HTTP_UPGRADE` header to the requested protocol.
+					if protocol = request.protocol and request.version.start_with?("HTTP/1")
+						env[CGI::HTTP_UPGRADE] = Array(protocol).join(",")
+					end
+					
+					if request.respond_to?(:hijack?) and request.hijack?
+						env[RACK_IS_HIJACK] = true
+						env[RACK_HIJACK] = proc{request.hijack!.io.dup}
+					end
+					
 					# HTTP/2 prefers `:authority` over `host`, so we do this for backwards compatibility.
 					env[CGI::HTTP_HOST] ||= request.authority
 								
@@ -106,7 +125,7 @@ module Protocol
 					
 					return Response.wrap(env, status, headers, meta, body, request)
 				rescue => exception
-					Console.logger.error(self) {exception}
+					Console.error(self, exception)
 					
 					body&.close if body.respond_to?(:close)
 					
@@ -123,6 +142,20 @@ module Protocol
 				def failure_response(exception)
 					Protocol::HTTP::Response.for_exception(exception)
 				end
+				
+				def self.extract_protocol(env, response, headers)
+					if protocol = response.protocol
+						# This is the newer mechanism for protocol upgrade:
+						if env["rack.protocol"]
+							headers["rack.protocol"] = protocol
+						
+						# Older mechanism for protocol upgrade:
+						elsif env[CGI::HTTP_UPGRADE]
+							headers["upgrade"] = protocol
+							headers["connection"] = "upgrade"
+						end
+					end
+				end
 			end
 		end
 	end
