@@ -1,10 +1,10 @@
 # frozen_string_literal: true
 
 # Released under the MIT License.
-# Copyright, 2022, by Samuel Williams.
+# Copyright, 2022-2024, by Samuel Williams.
 
-require 'protocol/http/body/readable'
-require 'protocol/http/body/stream'
+require "protocol/http/body/readable"
+require "protocol/http/body/stream"
 
 module Protocol
 	module Rack
